@@ -466,6 +466,17 @@ enum ccode { /* condition code names */
     C_none = -1
 };
 
+static inline uint8_t get_cond_opcode(enum ccode c)
+{
+    static const uint8_t ccode_opcodes[] = {
+        0x7, 0x3, 0x2, 0x6, 0x2, 0x4, 0xf, 0xd, 0xc, 0xe, 0x6, 0x2,
+        0x3, 0x7, 0x3, 0x5, 0xe, 0xc, 0xd, 0xf, 0x1, 0xb, 0x9, 0x5,
+        0x0, 0xa, 0xa, 0xb, 0x8, 0x4
+    };
+
+	return ccode_opcodes[(int)c];
+}
+
 /*
  * REX flags
  */
@@ -496,10 +507,24 @@ enum vex_class {
 enum prefixes { /* instruction prefixes */
     P_none = 0,
     PREFIX_ENUM_START = REG_ENUM_LIMIT,
-    P_A16 = PREFIX_ENUM_START, P_A32, P_A64, P_ASP,
-    P_LOCK, P_O16, P_O32, P_O64, P_OSP,
-    P_REP, P_REPE, P_REPNE, P_REPNZ, P_REPZ, P_TIMES,
-    P_WAIT, P_XACQUIRE, P_XRELEASE,
+    P_A16 = PREFIX_ENUM_START,
+    P_A32,
+    P_A64,
+    P_ASP,
+    P_LOCK,
+    P_O16,
+    P_O32,
+    P_O64,
+    P_OSP,
+    P_REP,
+    P_REPE,
+    P_REPNE,
+    P_REPNZ,
+    P_REPZ,
+    P_TIMES,
+    P_WAIT,
+    P_XACQUIRE,
+    P_XRELEASE,
     PREFIX_ENUM_LIMIT
 };
 
@@ -560,7 +585,7 @@ enum ea_type {
     EA_INVALID,     /* Not a valid EA at all */
     EA_SCALAR,      /* Scalar EA */
     EA_XMMVSIB,     /* XMM vector EA */
-    EA_YMMVSIB,     /* XMM vector EA */
+    EA_YMMVSIB,     /* YMM vector EA */
 };
 
 /*
